/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Ant-Contrib project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Ant-Contrib project (http://sourceforge.net/projects/ant-contrib)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name Ant-Contrib must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact
 *    ant-contrib-developers@lists.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "Ant-Contrib"
 *    nor may "Ant-Contrib" appear in their names without prior written
 *    permission of the Ant-Contrib project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE ANT-CONTRIB PROJECT OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
package net.sf.antcontrib.walls;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.taskdefs.Move;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Mapper;
import org.apache.tools.ant.types.Mapper.MapperType;
/*
 * Created on Aug 24, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/**
 * FILL IN JAVADOC HERE
 *
 * @author Dean Hiller(dean@xsoftware.biz)
 */
public class CompileWithWalls extends Task {
    private boolean setWallsTwice = false;
    private boolean setJavacTwice = false;
    private Walls walls;
    private Javac javac;
    public Walls createWalls() {
        if (walls != null)
            setWallsTwice = true;
        walls = new Walls();
        return walls;
    }
    public Javac createJavac() {
        if (javac != null)
            setJavacTwice = true;
        javac = new Javac();
        return javac;
    }
    public void execute() throws BuildException {
        if (javac == null)
            throw new BuildException(
                "There must be a nested javac element",
                getLocation());
        else if (walls == null)
            throw new BuildException(
                "There must be a nested walls element",
                getLocation());
        else if (setWallsTwice)
            throw new BuildException(
                "compilewithwalls task only supports one nested walls element or one walls attribute",
                getLocation());
        else if (setJavacTwice)
            throw new BuildException(
                "compilewithwalls task only supports one nested javac element",
                getLocation());
        project.addTaskDefinition("SilentMove", SilentMove.class);
        Enumeration packs = walls.getPackagesToCompile();
        while (packs.hasMoreElements()) {
            Package toCompile = (Package)packs.nextElement();
            //log("Isolating package "+toCompile.getPackage());
            File destDir = javac.getDestdir();
            String[] list = javac.getSrcdir().list();
            if (list.length != 1)
                throw new BuildException(
                    "srcdir in javac task must contain one and only one "
                        + "source directory with the compilewithwalls task\n"
                        + "Your javac srcdir contains "
                        + list.length
                        + " source directories",
                    getLocation());
            File srcDir = getProject().resolveFile(list[0]);
            if (!srcDir.exists()) {
                throw new BuildException(
                    "srcdir \"" + srcDir.getPath() + "\" does not exist!",
                    getLocation());
            } else if (destDir == null)
                throw new BuildException(
                    "destdir was not specified in nested javac task",
                    getLocation());
            FileSet excludes =
                toCompile.getExcludesFileSet(project, getLocation());
            moveFiles(srcDir, excludes, true, true);
            moveFiles(destDir, excludes, true, false);
            try {
                log("Compiling package " + toCompile.getPackage());
                javac.setProject(project);
                System.out.println("setting compiler to modern");
                javac.setCompiler("modern");
                javac.perform();
            } finally {
                FileSet includes = new FileSet();
                //move temp files back to java files.
                moveFiles(srcDir, includes, false, true);
                moveFiles(destDir, includes, false, false);
            }
        }
    }
    /**
     * Move java or class files to temp files or moves the temp files
     * back to java or class files.  This must be done because javac
     * is too nice and sticks already compiled classes and ones depended
     * on in the classpath destroying the compile time wall.  This way,
     * we can keep the wall up.
     * 
     * @param srcDir Directory to copy files from/to(Usually the java files dir or the class files dir)
     * @param fileset The fileset of files to include in the move.
     * @param moveToTempFile true if moving src files to temp files, false if moving temp files
     * 					back to src files.
     * @param isJavaFiles true if we are moving .java files, false if we are moving .class files.
     */
    private void moveFiles(
        File srcDir,
        FileSet fileset,
        boolean moveToTempFile,
        boolean isJavaFiles) {
        fileset.setDir(srcDir);
        if (!srcDir.exists())
            throw new BuildException(
                "Directory=" + srcDir + " does not exist",
                getLocation());
        //NOTE: for some reason in ant if move is used like below, the ${build}
        // directory will be deleted, which makes no sense because we are just
        //trying to move class files to *.ant-tempfiles.  Must be a bug in the 
        //move mapper task.  Need to check on this.  Until then we put a hack
        //in which I hate very much creating a temp file in the srcDir and then 
        //removing it after the move.
        //  <move todir="${build}">
        //    <fileset dir="${build}">
        //	    <include name="**/*.class"/>
        //    </fileset>
        //    <mapper type="glob" from="*.class" to="*.ant-tempfile"/>
        //  </move>
        //TODO: fix the move bug, or find out what is wrong with move and remove this hack.
        File tempFile = null;
        try {
            tempFile = File.createTempFile("antfile", null, srcDir);
        } catch (IOException e) {
            throw new BuildException(
                "A temporary file could not be created in " + srcDir,
                getLocation());
        }
        try {
            //before we do this, we have to move all files not
            //in the above fileset to xxx.java.ant-tempfile
            //so that they don't get dragged into the compile
            //This way we don't miss anything and all the dependencies
            //are listed or the compile will break.
            Move move = (Move)project.createTask("SilentMove");
            move.setProject(project);
            move.setOwningTarget(target);
            move.setTaskName(getTaskName());
            move.setLocation(getLocation());
            move.setTodir(srcDir);
            move.setOverwrite(true);
            move.addFileset(fileset);
            MapperType globType = new Mapper.MapperType();
            globType.setValue("glob");
            Mapper mapper = move.createMapper();
            mapper.setType(globType);
            String pattern;
            String tempPattern;
            if (isJavaFiles) {
                pattern = "*.java";
                tempPattern = "*.j-anttemp";
            } else {
                pattern = "*.class";
                tempPattern = "*.c-anttemp";
            }
            if (moveToTempFile) {
                log(
                    "moving " + pattern + " files to temp files",
                    Project.MSG_VERBOSE);
                mapper.setFrom(pattern);
                mapper.setTo(tempPattern);
            } else {
                log(
                    "moving temp files back to " + pattern + " files",
                    Project.MSG_VERBOSE);
                mapper.setFrom(tempPattern);
                mapper.setTo(pattern);
            }
            move.perform();
        } finally {
            if (tempFile != null)
                tempFile.delete();
        }
    }
}
