/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Ant-Contrib project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Ant-Contrib project (http://sourceforge.net/projects/ant-contrib)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name Ant-Contrib must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact
 *    ant-contrib-developers@lists.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "Ant-Contrib"
 *    nor may "Ant-Contrib" appear in their names without prior written
 *    permission of the Ant-Contrib project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE ANT-CONTRIB PROJECT OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
package net.sf.antcontrib.walls;

import java.io.File;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.ProjectHelper;
import org.apache.tools.ant.types.FileSet;


/*
 * Created on Aug 24, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/**
 * FILL IN JAVADOC HERE
 *
 * @author Dean Hiller(dean@xsoftware.biz)
 */
public class Package {
    
    private String name;
    private String pack;

    //signifies the package did not end with .* or .**
    private String failureReason = null;
    
    //holds the name attribute of the package element of each
    //package this package depends on.
    private String[] depends;
    //holds the package attribute of the package element for each
    //package this package depends on.
    private Vector dependsPack = new Vector();
    
    private FileSet set = new FileSet();

    public void setName(String name) {
        this.name = name;
    }
    
    public void setPackage(String pack) {
        this.pack = pack;
    }
    
    public void setDepends(String d) {
        //parse this first.
        StringTokenizer tok = new StringTokenizer(d, ", \t");
        depends = new String[tok.countTokens()];
        int i = 0;
        while(tok.hasMoreTokens()) {
            depends[i] = tok.nextToken();	
            i++;
        }
    }

    public String getName() {
        return name;
    }
    
    public String[] getDepends() {
        return depends;
    }
    
    /**
     * FILL IN JAVADOC HERE
     * 
     */
    public FileSet getExcludesFileSet(Project p, Location l) throws BuildException {
        
        if(failureReason != null)
            throw new BuildException(failureReason, l);
        else if(pack.indexOf("/") != -1 || pack.indexOf("\\") != -1)
            throw new BuildException("A package name cannot contain '\\' or '/' like package="+pack
                                +"\nIt must look like biz.xsoftware.* for example", l);

        String propertiesResolved = ProjectHelper.replaceProperties(p, pack);
        
        
        propertiesResolved = propertiesResolved.replace('.', File.separatorChar);

        if(propertiesResolved.endsWith("**"))
            propertiesResolved += "/*.java";
        else if(propertiesResolved.endsWith("*"))
            propertiesResolved += ".java";
        else
            throw new RuntimeException("Please report this bug");
        
        //first exclude the compilation module, then exclude all it's
        //dependencies too.
        set.setExcludes(propertiesResolved);
        
        for(int i = 0; i < dependsPack.size(); i++) {
            String s = (String)dependsPack.elementAt(i);
            s = ProjectHelper.replaceProperties(p, s);
            s = s.replace('.', File.separatorChar);
            
            //must be a good package here because we would have already
            //compiled it.			
            set.setExcludes(s);
        }
        
//		set.setIncludes("**/*.java");
//		set.setIncludes("**/*.class");

        return set;
    }

    /**
     * FILL IN JAVADOC HERE
     * @param dependsPackage
     */
    public void addExcludesElement(String dependsPackage) {
        dependsPack.addElement(dependsPackage);
    }

    /**
     * FILL IN JAVADOC HERE
     * @return
     */
    public String getPackage() {
        return pack;
    }

    /**
     * FILL IN JAVADOC HERE
     * @param r
     */
    public void setFaultReason(String r) {
        failureReason = r;
    }
}
